import inquirer
import typer
from terminaltables import AsciiTable

from savvihub.cli.exceptions import ExitException
from savvihub.cli.inputs.ssh import ssh_public_key_value_callback, ssh_key_name_callback
from savvihub.cli.typer import Typer, Context
from savvihub.common.utils import parse_time_to_ago

ssh_keys_app = Typer()


@ssh_keys_app.callback()
def main():
    """
    Manage ssh public keys
    """


@ssh_keys_app.command(user_required=True)
def add(
    ctx: Context,
    ssh_public_key_value: str = typer.Option(None, '--key-path', callback=ssh_public_key_value_callback, help='SSH key path.'),
    name: str = typer.Option(None, '--name', callback=ssh_key_name_callback, help='SSH key name.'),
):
    """
    Add a ssh public key
    """
    ctx.authenticated_client.ssh_key_add(ssh_public_key_value, name, ctx.store.get('ssh_key_filename'))
    typer.echo(f'\n'
               f'Successfully added.')


@ssh_keys_app.command(user_required=True)
def list(ctx: Context):
    """
    List ssh public keys
    """
    ssh_keys_response = ctx.authenticated_client.ssh_key_list()

    table = AsciiTable([
        ['NAME', 'FINGERPRINT', 'CREATED'],
        *[[k.name, k.fingerprint, parse_time_to_ago(k.created_dt)]
          for k in ssh_keys_response.ssh_keys],
    ])
    table.inner_column_border = False
    table.inner_heading_row_border = False
    table.inner_footing_row_border = False
    table.outer_border = False

    typer.echo(table.table)


@ssh_keys_app.command(user_required=True)
def delete(ctx: Context):
    """
    Delete a ssh public key
    """
    ssh_keys_response = ctx.authenticated_client.ssh_key_list()
    if len(ssh_keys_response.ssh_keys) == 0:
        raise ExitException('SSH public key not found.')

    ssh_key = inquirer.prompt([inquirer.List(
        'ssh_key',
        message='Select ssh public key',
        choices=[(f'{k.name} / {k.fingerprint} (created {parse_time_to_ago(k.created_dt)})', k) for k in ssh_keys_response.ssh_keys],
    )], raise_keyboard_interrupt=True).get('ssh_key')

    ctx.authenticated_client.ssh_key_delete(ssh_key.id)
    typer.echo('Successfully deleted.')



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseProject(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cached_git_default_branch': 'str',
        'cached_git_http_url_to_repo': 'str',
        'cached_git_owner_slug': 'str',
        'cached_git_repo_slug': 'str',
        'cached_git_ssh_url_to_repo': 'str',
        'created_dt': 'datetime',
        'description': 'str',
        'git_provider': 'str',
        'id': 'int',
        'immutable_slug': 'str',
        'is_public': 'bool',
        'name': 'str',
        'organization_id': 'int',
        'primary_owner_id': 'int',
        'project_organization': 'int',
        'project_primary_owner': 'int',
        'project_volume': 'int',
        'type': 'str',
        'updated_dt': 'datetime',
        'volume_id': 'int'
    }

    attribute_map = {
        'cached_git_default_branch': 'cached_git_default_branch',
        'cached_git_http_url_to_repo': 'cached_git_http_url_to_repo',
        'cached_git_owner_slug': 'cached_git_owner_slug',
        'cached_git_repo_slug': 'cached_git_repo_slug',
        'cached_git_ssh_url_to_repo': 'cached_git_ssh_url_to_repo',
        'created_dt': 'created_dt',
        'description': 'description',
        'git_provider': 'git_provider',
        'id': 'id',
        'immutable_slug': 'immutable_slug',
        'is_public': 'is_public',
        'name': 'name',
        'organization_id': 'organization_id',
        'primary_owner_id': 'primary_owner_id',
        'project_organization': 'project_organization',
        'project_primary_owner': 'project_primary_owner',
        'project_volume': 'project_volume',
        'type': 'type',
        'updated_dt': 'updated_dt',
        'volume_id': 'volume_id'
    }

    def __init__(self, cached_git_default_branch=None, cached_git_http_url_to_repo=None, cached_git_owner_slug=None, cached_git_repo_slug=None, cached_git_ssh_url_to_repo=None, created_dt=None, description=None, git_provider=None, id=None, immutable_slug=None, is_public=None, name=None, organization_id=None, primary_owner_id=None, project_organization=None, project_primary_owner=None, project_volume=None, type=None, updated_dt=None, volume_id=None, local_vars_configuration=None):  # noqa: E501
        """ResponseProject - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cached_git_default_branch = None
        self._cached_git_http_url_to_repo = None
        self._cached_git_owner_slug = None
        self._cached_git_repo_slug = None
        self._cached_git_ssh_url_to_repo = None
        self._created_dt = None
        self._description = None
        self._git_provider = None
        self._id = None
        self._immutable_slug = None
        self._is_public = None
        self._name = None
        self._organization_id = None
        self._primary_owner_id = None
        self._project_organization = None
        self._project_primary_owner = None
        self._project_volume = None
        self._type = None
        self._updated_dt = None
        self._volume_id = None
        self.discriminator = None

        self.cached_git_default_branch = cached_git_default_branch
        self.cached_git_http_url_to_repo = cached_git_http_url_to_repo
        self.cached_git_owner_slug = cached_git_owner_slug
        self.cached_git_repo_slug = cached_git_repo_slug
        self.cached_git_ssh_url_to_repo = cached_git_ssh_url_to_repo
        self.created_dt = created_dt
        self.description = description
        self.git_provider = git_provider
        self.id = id
        self.immutable_slug = immutable_slug
        self.is_public = is_public
        self.name = name
        self.organization_id = organization_id
        self.primary_owner_id = primary_owner_id
        self.project_organization = project_organization
        self.project_primary_owner = project_primary_owner
        self.project_volume = project_volume
        self.type = type
        self.updated_dt = updated_dt
        self.volume_id = volume_id

    @property
    def cached_git_default_branch(self):
        """Gets the cached_git_default_branch of this ResponseProject.  # noqa: E501


        :return: The cached_git_default_branch of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._cached_git_default_branch

    @cached_git_default_branch.setter
    def cached_git_default_branch(self, cached_git_default_branch):
        """Sets the cached_git_default_branch of this ResponseProject.


        :param cached_git_default_branch: The cached_git_default_branch of this ResponseProject.  # noqa: E501
        :type cached_git_default_branch: str
        """
        if self.local_vars_configuration.client_side_validation and cached_git_default_branch is None:  # noqa: E501
            raise ValueError("Invalid value for `cached_git_default_branch`, must not be `None`")  # noqa: E501

        self._cached_git_default_branch = cached_git_default_branch

    @property
    def cached_git_http_url_to_repo(self):
        """Gets the cached_git_http_url_to_repo of this ResponseProject.  # noqa: E501


        :return: The cached_git_http_url_to_repo of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._cached_git_http_url_to_repo

    @cached_git_http_url_to_repo.setter
    def cached_git_http_url_to_repo(self, cached_git_http_url_to_repo):
        """Sets the cached_git_http_url_to_repo of this ResponseProject.


        :param cached_git_http_url_to_repo: The cached_git_http_url_to_repo of this ResponseProject.  # noqa: E501
        :type cached_git_http_url_to_repo: str
        """

        self._cached_git_http_url_to_repo = cached_git_http_url_to_repo

    @property
    def cached_git_owner_slug(self):
        """Gets the cached_git_owner_slug of this ResponseProject.  # noqa: E501


        :return: The cached_git_owner_slug of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._cached_git_owner_slug

    @cached_git_owner_slug.setter
    def cached_git_owner_slug(self, cached_git_owner_slug):
        """Sets the cached_git_owner_slug of this ResponseProject.


        :param cached_git_owner_slug: The cached_git_owner_slug of this ResponseProject.  # noqa: E501
        :type cached_git_owner_slug: str
        """
        if self.local_vars_configuration.client_side_validation and cached_git_owner_slug is None:  # noqa: E501
            raise ValueError("Invalid value for `cached_git_owner_slug`, must not be `None`")  # noqa: E501

        self._cached_git_owner_slug = cached_git_owner_slug

    @property
    def cached_git_repo_slug(self):
        """Gets the cached_git_repo_slug of this ResponseProject.  # noqa: E501


        :return: The cached_git_repo_slug of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._cached_git_repo_slug

    @cached_git_repo_slug.setter
    def cached_git_repo_slug(self, cached_git_repo_slug):
        """Sets the cached_git_repo_slug of this ResponseProject.


        :param cached_git_repo_slug: The cached_git_repo_slug of this ResponseProject.  # noqa: E501
        :type cached_git_repo_slug: str
        """
        if self.local_vars_configuration.client_side_validation and cached_git_repo_slug is None:  # noqa: E501
            raise ValueError("Invalid value for `cached_git_repo_slug`, must not be `None`")  # noqa: E501

        self._cached_git_repo_slug = cached_git_repo_slug

    @property
    def cached_git_ssh_url_to_repo(self):
        """Gets the cached_git_ssh_url_to_repo of this ResponseProject.  # noqa: E501


        :return: The cached_git_ssh_url_to_repo of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._cached_git_ssh_url_to_repo

    @cached_git_ssh_url_to_repo.setter
    def cached_git_ssh_url_to_repo(self, cached_git_ssh_url_to_repo):
        """Sets the cached_git_ssh_url_to_repo of this ResponseProject.


        :param cached_git_ssh_url_to_repo: The cached_git_ssh_url_to_repo of this ResponseProject.  # noqa: E501
        :type cached_git_ssh_url_to_repo: str
        """

        self._cached_git_ssh_url_to_repo = cached_git_ssh_url_to_repo

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseProject.  # noqa: E501


        :return: The created_dt of this ResponseProject.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseProject.


        :param created_dt: The created_dt of this ResponseProject.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def description(self):
        """Gets the description of this ResponseProject.  # noqa: E501


        :return: The description of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ResponseProject.


        :param description: The description of this ResponseProject.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def git_provider(self):
        """Gets the git_provider of this ResponseProject.  # noqa: E501


        :return: The git_provider of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._git_provider

    @git_provider.setter
    def git_provider(self, git_provider):
        """Sets the git_provider of this ResponseProject.


        :param git_provider: The git_provider of this ResponseProject.  # noqa: E501
        :type git_provider: str
        """
        if self.local_vars_configuration.client_side_validation and git_provider is None:  # noqa: E501
            raise ValueError("Invalid value for `git_provider`, must not be `None`")  # noqa: E501

        self._git_provider = git_provider

    @property
    def id(self):
        """Gets the id of this ResponseProject.  # noqa: E501


        :return: The id of this ResponseProject.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseProject.


        :param id: The id of this ResponseProject.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def immutable_slug(self):
        """Gets the immutable_slug of this ResponseProject.  # noqa: E501


        :return: The immutable_slug of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._immutable_slug

    @immutable_slug.setter
    def immutable_slug(self, immutable_slug):
        """Sets the immutable_slug of this ResponseProject.


        :param immutable_slug: The immutable_slug of this ResponseProject.  # noqa: E501
        :type immutable_slug: str
        """
        if self.local_vars_configuration.client_side_validation and immutable_slug is None:  # noqa: E501
            raise ValueError("Invalid value for `immutable_slug`, must not be `None`")  # noqa: E501

        self._immutable_slug = immutable_slug

    @property
    def is_public(self):
        """Gets the is_public of this ResponseProject.  # noqa: E501


        :return: The is_public of this ResponseProject.  # noqa: E501
        :rtype: bool
        """
        return self._is_public

    @is_public.setter
    def is_public(self, is_public):
        """Sets the is_public of this ResponseProject.


        :param is_public: The is_public of this ResponseProject.  # noqa: E501
        :type is_public: bool
        """
        if self.local_vars_configuration.client_side_validation and is_public is None:  # noqa: E501
            raise ValueError("Invalid value for `is_public`, must not be `None`")  # noqa: E501

        self._is_public = is_public

    @property
    def name(self):
        """Gets the name of this ResponseProject.  # noqa: E501


        :return: The name of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ResponseProject.


        :param name: The name of this ResponseProject.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def organization_id(self):
        """Gets the organization_id of this ResponseProject.  # noqa: E501


        :return: The organization_id of this ResponseProject.  # noqa: E501
        :rtype: int
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this ResponseProject.


        :param organization_id: The organization_id of this ResponseProject.  # noqa: E501
        :type organization_id: int
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def primary_owner_id(self):
        """Gets the primary_owner_id of this ResponseProject.  # noqa: E501


        :return: The primary_owner_id of this ResponseProject.  # noqa: E501
        :rtype: int
        """
        return self._primary_owner_id

    @primary_owner_id.setter
    def primary_owner_id(self, primary_owner_id):
        """Sets the primary_owner_id of this ResponseProject.


        :param primary_owner_id: The primary_owner_id of this ResponseProject.  # noqa: E501
        :type primary_owner_id: int
        """
        if self.local_vars_configuration.client_side_validation and primary_owner_id is None:  # noqa: E501
            raise ValueError("Invalid value for `primary_owner_id`, must not be `None`")  # noqa: E501

        self._primary_owner_id = primary_owner_id

    @property
    def project_organization(self):
        """Gets the project_organization of this ResponseProject.  # noqa: E501


        :return: The project_organization of this ResponseProject.  # noqa: E501
        :rtype: int
        """
        return self._project_organization

    @project_organization.setter
    def project_organization(self, project_organization):
        """Sets the project_organization of this ResponseProject.


        :param project_organization: The project_organization of this ResponseProject.  # noqa: E501
        :type project_organization: int
        """
        if self.local_vars_configuration.client_side_validation and project_organization is None:  # noqa: E501
            raise ValueError("Invalid value for `project_organization`, must not be `None`")  # noqa: E501

        self._project_organization = project_organization

    @property
    def project_primary_owner(self):
        """Gets the project_primary_owner of this ResponseProject.  # noqa: E501


        :return: The project_primary_owner of this ResponseProject.  # noqa: E501
        :rtype: int
        """
        return self._project_primary_owner

    @project_primary_owner.setter
    def project_primary_owner(self, project_primary_owner):
        """Sets the project_primary_owner of this ResponseProject.


        :param project_primary_owner: The project_primary_owner of this ResponseProject.  # noqa: E501
        :type project_primary_owner: int
        """
        if self.local_vars_configuration.client_side_validation and project_primary_owner is None:  # noqa: E501
            raise ValueError("Invalid value for `project_primary_owner`, must not be `None`")  # noqa: E501

        self._project_primary_owner = project_primary_owner

    @property
    def project_volume(self):
        """Gets the project_volume of this ResponseProject.  # noqa: E501


        :return: The project_volume of this ResponseProject.  # noqa: E501
        :rtype: int
        """
        return self._project_volume

    @project_volume.setter
    def project_volume(self, project_volume):
        """Sets the project_volume of this ResponseProject.


        :param project_volume: The project_volume of this ResponseProject.  # noqa: E501
        :type project_volume: int
        """
        if self.local_vars_configuration.client_side_validation and project_volume is None:  # noqa: E501
            raise ValueError("Invalid value for `project_volume`, must not be `None`")  # noqa: E501

        self._project_volume = project_volume

    @property
    def type(self):
        """Gets the type of this ResponseProject.  # noqa: E501


        :return: The type of this ResponseProject.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ResponseProject.


        :param type: The type of this ResponseProject.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseProject.  # noqa: E501


        :return: The updated_dt of this ResponseProject.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseProject.


        :param updated_dt: The updated_dt of this ResponseProject.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def volume_id(self):
        """Gets the volume_id of this ResponseProject.  # noqa: E501


        :return: The volume_id of this ResponseProject.  # noqa: E501
        :rtype: int
        """
        return self._volume_id

    @volume_id.setter
    def volume_id(self, volume_id):
        """Sets the volume_id of this ResponseProject.


        :param volume_id: The volume_id of this ResponseProject.  # noqa: E501
        :type volume_id: int
        """
        if self.local_vars_configuration.client_side_validation and volume_id is None:  # noqa: E501
            raise ValueError("Invalid value for `volume_id`, must not be `None`")  # noqa: E501

        self._volume_id = volume_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseProject):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseProject):
            return True

        return self.to_dict() != other.to_dict()



def parse_bytes(B):
    B = float(B)
    KB = float(1024)
    MB = float(KB ** 2)
    GB = float(KB ** 3)
    TB = float(KB ** 4)

    if B < KB:
        return '{0} {1}'.format(B,'Bytes' if 0 == B > 1 else 'Byte')
    elif KB <= B < MB:
        return '{0:.2f} KB'.format(B/KB)
    elif MB <= B < GB:
        return '{0:.2f} MB'.format(B/MB)
    elif GB <= B < TB:
        return '{0:.2f} GB'.format(B/GB)
    elif TB <= B:
        return '{0:.2f} TB'.format(B/TB)


from typing import List

import inquirer
import typer

from savvihub.cli.exceptions import ExitException
from savvihub.cli.typer import Context
from savvihub.cli.utils import find_from_inquirer


def image_url_callback(ctx: Context, image_url: str) -> str:
    assert ctx.params['organization_name'] and ctx.params['processor_type']

    images = ctx.authenticated_client.kernel_image_list(ctx.params['organization_name']).results
    images = [i for i in images if i.processor_type == ctx.params['processor_type']]
    if image_url:
        for image in images:
            if image.image_url == image_url:
                return image_url
        else:
            raise ExitException(f'Image not found: {image_url}')

    return find_from_inquirer(
        images,
        lambda x: f'{x.image_url} ({x.name})',
        "Please choose a kernel image"
    ).image_url

def env_vars_callback(ctx: Context, env_vars: List[str]) -> List[str]:
    ctx.store['env_vars'] = []
    for env_var in env_vars:
        try:
            env_key, env_value = env_var.split("=", 1)
            ctx.store['env_vars'].append({
                'key': env_key,
                'value': env_value,
            })
        except ValueError:
            raise ExitException(f'Cannot parse environment variable: {env_var}')

    return env_vars


def cluster_name_callback(ctx: Context, cluster_name: str) -> str:
    assert ctx.params['organization_name']

    clusters = [c for c in ctx.authenticated_client.cluster_list(ctx.params['organization_name']).clusters
                if c.status == 'connected']
    if cluster_name:
        for cluster in clusters:
            if cluster.name == cluster_name.strip():
                ctx.store['cluster'] = cluster
                return cluster.name
        else:
            raise ExitException(f'Cluster not found: {cluster_name.strip()}')

    elif len(clusters) == 1:
        typer.echo(f'The cluster is automatically set to `{clusters[0].name}{" (SavviHub)" if clusters[0].is_savvihub_managed else f" (Custom)"}`.')
        ctx.store['cluster'] = clusters[0]
        return clusters[0].name

    else:
        selected_cluster = find_from_inquirer(
            clusters,
            lambda x: f'{x.name}{" (SavviHub)" if x.is_savvihub_managed else f" ({x.name})"}',
            "Please choose a cluster"
        )
        ctx.store['cluster'] = selected_cluster

    return selected_cluster.name

def start_command_callback(start_command: str) -> str:
    if start_command:
        return start_command

    return inquirer.prompt([inquirer.Text(
        'start_command',
        message="Start command",
        default="python main.py",
    )], raise_keyboard_interrupt=True).get('start_command')



import os

from requests_futures.sessions import FuturesSession

from savvihub.api.file_object import DownloadableFileObject
from savvihub.common.utils import wait_all_futures


class Downloader:
    @classmethod
    def download(cls, local_path, file, progressable=None):
        dirname = os.path.dirname(local_path)
        basename = os.path.basename(local_path)
        d = DownloadableFileObject(file.download_url.url, dirname, basename)
        d.download(progressable=progressable)

    @classmethod
    def bulk_download(cls, local_base_path, remote_files, progressable=None):
        if len(remote_files) <= 0:
            return
        session = FuturesSession(max_workers=os.environ.get('SAVVIHUB_PARALLEL', 20))
        futures = []
        for remote_file in remote_files:
            if remote_file.path.endswith('/'):
                continue

            d = DownloadableFileObject(remote_file.download_url.url, local_base_path, remote_file.path)
            futures.append(d.download(session, progressable=progressable))

        wait_all_futures(futures)


# coding: utf-8

# flake8: noqa
"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from openapi_client.models.access_token_create_api_payload import AccessTokenCreateAPIPayload
from openapi_client.models.account_access_token_create_response import AccountAccessTokenCreateResponse
from openapi_client.models.account_invitation_token_validate_response import AccountInvitationTokenValidateResponse
from openapi_client.models.account_ssh_key_list_response import AccountSSHKeyListResponse
from openapi_client.models.account_sign_in_cli_check_response import AccountSignInCliCheckResponse
from openapi_client.models.account_sign_in_cli_token_response import AccountSignInCliTokenResponse
from openapi_client.models.cli_project_create_api_payload import CLIProjectCreateAPIPayload
from openapi_client.models.change_password_api_payload import ChangePasswordAPIPayload
from openapi_client.models.change_username_api_payload import ChangeUsernameAPIPayload
from openapi_client.models.cli_experiment_plots_files_update_api_payload import CliExperimentPlotsFilesUpdateAPIPayload
from openapi_client.models.cli_experiment_plots_metrics_update_api_payload import CliExperimentPlotsMetricsUpdateAPIPayload
from openapi_client.models.cluster_all_cluster_list_response import ClusterAllClusterListResponse
from openapi_client.models.cluster_custom_cluster_key_response import ClusterCustomClusterKeyResponse
from openapi_client.models.cluster_custom_cluster_list_response import ClusterCustomClusterListResponse
from openapi_client.models.cluster_custom_cluster_node_list_response import ClusterCustomClusterNodeListResponse
from openapi_client.models.custom_cluster_add_api_payload import CustomClusterAddAPIPayload
from openapi_client.models.custom_cluster_update_api_payload import CustomClusterUpdateAPIPayload
from openapi_client.models.dataset_summary_update_api_payload import DatasetSummaryUpdateAPIPayload
from openapi_client.models.dataset_update_api_payload import DatasetUpdateAPIPayload
from openapi_client.models.dataset_version_create_api_payload import DatasetVersionCreateAPIPayload
from openapi_client.models.dataset_version_update_api_payload import DatasetVersionUpdateAPIPayload
from openapi_client.models.datasetversion_dataset_version_list_response import DatasetversionDatasetVersionListResponse
from openapi_client.models.experiment_create_api_payload import ExperimentCreateAPIPayload
from openapi_client.models.experiment_experiment_list_by_project_response import ExperimentExperimentListByProjectResponse
from openapi_client.models.experiment_experiment_list_response import ExperimentExperimentListResponse
from openapi_client.models.experiment_experiment_logs_response import ExperimentExperimentLogsResponse
from openapi_client.models.experiment_experiment_parameter_list_response import ExperimentExperimentParameterListResponse
from openapi_client.models.experiment_experiment_plots_response import ExperimentExperimentPlotsResponse
from openapi_client.models.experiment_experiment_system_metrics_response import ExperimentExperimentSystemMetricsResponse
from openapi_client.models.experiment_sidecar_experiment_idle_response import ExperimentSidecarExperimentIdleResponse
from openapi_client.models.experiment_update_api_payload import ExperimentUpdateAPIPayload
from openapi_client.models.external_cli_git_hub_token_response import ExternalCliGitHubTokenResponse
from openapi_client.models.external_git_hub_config_response import ExternalGitHubConfigResponse
from openapi_client.models.external_git_hub_repository_list_response import ExternalGitHubRepositoryListResponse
from openapi_client.models.external_google_config_response import ExternalGoogleConfigResponse
from openapi_client.models.external_slack_config_response import ExternalSlackConfigResponse
from openapi_client.models.gs_dataset_create_api_payload import GSDatasetCreateAPIPayload
from openapi_client.models.git_hub_authorize_api_payload import GitHubAuthorizeAPIPayload
from openapi_client.models.git_hub_project_create_api_payload import GitHubProjectCreateAPIPayload
from openapi_client.models.github_code_of_conduct import GithubCodeOfConduct
from openapi_client.models.github_license import GithubLicense
from openapi_client.models.github_match import GithubMatch
from openapi_client.models.github_organization import GithubOrganization
from openapi_client.models.github_plan import GithubPlan
from openapi_client.models.github_repository import GithubRepository
from openapi_client.models.github_text_match import GithubTextMatch
from openapi_client.models.github_user import GithubUser
from openapi_client.models.invitation_token_validate_api_payload import InvitationTokenValidateAPIPayload
from openapi_client.models.kernel_kernel_image_list_response import KernelKernelImageListResponse
from openapi_client.models.kernel_kernel_resource_spec_list_response import KernelKernelResourceSpecListResponse
from openapi_client.models.local_nfs_dataset_create_api_payload import LocalNFSDatasetCreateAPIPayload
from openapi_client.models.model_access_token import ModelAccessToken
from openapi_client.models.model_access_token_edges import ModelAccessTokenEdges
from openapi_client.models.model_billing_history import ModelBillingHistory
from openapi_client.models.model_billing_history_edges import ModelBillingHistoryEdges
from openapi_client.models.model_cli_driven_project import ModelCLIDrivenProject
from openapi_client.models.model_env_var import ModelEnvVar
from openapi_client.models.model_env_vars import ModelEnvVars
from openapi_client.models.model_experiment_metrics_summary import ModelExperimentMetricsSummary
from openapi_client.models.model_file_metadata import ModelFileMetadata
from openapi_client.models.model_kernel_cluster import ModelKernelCluster
from openapi_client.models.model_kernel_cluster_config import ModelKernelClusterConfig
from openapi_client.models.model_kernel_cluster_config_ingress import ModelKernelClusterConfigIngress
from openapi_client.models.model_kernel_cluster_config_service import ModelKernelClusterConfigService
from openapi_client.models.model_kernel_cluster_config_storage_class import ModelKernelClusterConfigStorageClass
from openapi_client.models.model_kernel_cluster_edges import ModelKernelClusterEdges
from openapi_client.models.model_kernel_cluster_node import ModelKernelClusterNode
from openapi_client.models.model_kernel_cluster_node_edges import ModelKernelClusterNodeEdges
from openapi_client.models.model_kernel_cluster_select_policies import ModelKernelClusterSelectPolicies
from openapi_client.models.model_kernel_cluster_select_policy import ModelKernelClusterSelectPolicy
from openapi_client.models.model_kernel_cluster_storage import ModelKernelClusterStorage
from openapi_client.models.model_kernel_cluster_storage_edges import ModelKernelClusterStorageEdges
from openapi_client.models.model_kernel_image import ModelKernelImage
from openapi_client.models.model_kernel_image_edges import ModelKernelImageEdges
from openapi_client.models.model_kernel_resource_spec import ModelKernelResourceSpec
from openapi_client.models.model_kernel_resource_spec_field import ModelKernelResourceSpecField
from openapi_client.models.model_kernel_resource_spec_gpu_labels import ModelKernelResourceSpecGpuLabels
from openapi_client.models.model_notification_config import ModelNotificationConfig
from openapi_client.models.model_notification_config_edges import ModelNotificationConfigEdges
from openapi_client.models.model_organization import ModelOrganization
from openapi_client.models.model_organization_credentials import ModelOrganizationCredentials
from openapi_client.models.model_organization_credentials_edges import ModelOrganizationCredentialsEdges
from openapi_client.models.model_organization_edges import ModelOrganizationEdges
from openapi_client.models.model_page_info_with_count import ModelPageInfoWithCount
from openapi_client.models.model_parameter import ModelParameter
from openapi_client.models.model_permission_data import ModelPermissionData
from openapi_client.models.model_permission_detail import ModelPermissionDetail
from openapi_client.models.model_permissions import ModelPermissions
from openapi_client.models.model_project import ModelProject
from openapi_client.models.model_project_edges import ModelProjectEdges
from openapi_client.models.model_range import ModelRange
from openapi_client.models.model_storage import ModelStorage
from openapi_client.models.model_storage_edges import ModelStorageEdges
from openapi_client.models.model_sweep import ModelSweep
from openapi_client.models.model_sweep_edges import ModelSweepEdges
from openapi_client.models.model_sweep_objective import ModelSweepObjective
from openapi_client.models.model_sweep_search_space import ModelSweepSearchSpace
from openapi_client.models.model_user import ModelUser
from openapi_client.models.model_user_edges import ModelUserEdges
from openapi_client.models.model_user_organization import ModelUserOrganization
from openapi_client.models.model_user_organization_edges import ModelUserOrganizationEdges
from openapi_client.models.model_version_control_project import ModelVersionControlProject
from openapi_client.models.model_volume import ModelVolume
from openapi_client.models.model_volume_edges import ModelVolumeEdges
from openapi_client.models.model_volume_mount import ModelVolumeMount
from openapi_client.models.model_volume_mount_field import ModelVolumeMountField
from openapi_client.models.model_volume_source_dataset import ModelVolumeSourceDataset
from openapi_client.models.model_volume_source_host_path import ModelVolumeSourceHostPath
from openapi_client.models.model_volume_source_nfs import ModelVolumeSourceNFS
from openapi_client.models.model_volume_source_project import ModelVolumeSourceProject
from openapi_client.models.model_volume_source_volume import ModelVolumeSourceVolume
from openapi_client.models.model_workload import ModelWorkload
from openapi_client.models.model_workload_edges import ModelWorkloadEdges
from openapi_client.models.model_workload_endpoint import ModelWorkloadEndpoint
from openapi_client.models.model_workload_endpoints import ModelWorkloadEndpoints
from openapi_client.models.model_workload_history import ModelWorkloadHistory
from openapi_client.models.model_workload_history_edges import ModelWorkloadHistoryEdges
from openapi_client.models.model_workspace_port import ModelWorkspacePort
from openapi_client.models.model_workspace_ports import ModelWorkspacePorts
from openapi_client.models.organization_aws_credentials_add_api_payload import OrganizationAWSCredentialsAddAPIPayload
from openapi_client.models.organization_create_api_payload import OrganizationCreateAPIPayload
from openapi_client.models.organization_docker_credentials_add_api_payload import OrganizationDockerCredentialsAddAPIPayload
from openapi_client.models.organization_member_add_api_payload import OrganizationMemberAddAPIPayload
from openapi_client.models.organization_notification_config_email_add_api_payload import OrganizationNotificationConfigEmailAddAPIPayload
from openapi_client.models.organization_organization_billing_history_filter_list_response import OrganizationOrganizationBillingHistoryFilterListResponse
from openapi_client.models.organization_organization_billing_history_list_response import OrganizationOrganizationBillingHistoryListResponse
from openapi_client.models.organization_organization_billing_summary_response import OrganizationOrganizationBillingSummaryResponse
from openapi_client.models.organization_organization_billing_total_response import OrganizationOrganizationBillingTotalResponse
from openapi_client.models.organization_organization_list_response import OrganizationOrganizationListResponse
from openapi_client.models.organization_organization_member_list_response import OrganizationOrganizationMemberListResponse
from openapi_client.models.organization_ssh_credentials_add_api_payload import OrganizationSSHCredentialsAddAPIPayload
from openapi_client.models.organization_update_api_payload import OrganizationUpdateAPIPayload
from openapi_client.models.project_project_branch_list_response import ProjectProjectBranchListResponse
from openapi_client.models.project_project_commit_list_response import ProjectProjectCommitListResponse
from openapi_client.models.project_update_api_payload import ProjectUpdateAPIPayload
from openapi_client.models.proto_branch import ProtoBranch
from openapi_client.models.proto_commit import ProtoCommit
from openapi_client.models.proto_commit_author import ProtoCommitAuthor
from openapi_client.models.proto_current_system_metric import ProtoCurrentSystemMetric
from openapi_client.models.proto_experiment_plot_file import ProtoExperimentPlotFile
from openapi_client.models.proto_experiment_plot_metric import ProtoExperimentPlotMetric
from openapi_client.models.proto_metric import ProtoMetric
from openapi_client.models.proto_system_metric import ProtoSystemMetric
from openapi_client.models.proto_system_metric_list import ProtoSystemMetricList
from openapi_client.models.proto_volume_mount_request import ProtoVolumeMountRequest
from openapi_client.models.proto_volume_mount_request_source_cli_driven_project import ProtoVolumeMountRequestSourceCLIDrivenProject
from openapi_client.models.proto_volume_mount_request_source_dataset import ProtoVolumeMountRequestSourceDataset
from openapi_client.models.proto_volume_mount_request_source_host_path import ProtoVolumeMountRequestSourceHostPath
from openapi_client.models.proto_volume_mount_request_source_nfs import ProtoVolumeMountRequestSourceNFS
from openapi_client.models.proto_volume_mount_request_source_project import ProtoVolumeMountRequestSourceProject
from openapi_client.models.proto_volume_mount_request_source_version_control_project import ProtoVolumeMountRequestSourceVersionControlProject
from openapi_client.models.proto_volume_mount_request_source_volume import ProtoVolumeMountRequestSourceVolume
from openapi_client.models.proto_volume_mount_requests import ProtoVolumeMountRequests
from openapi_client.models.proto_workload_log import ProtoWorkloadLog
from openapi_client.models.region_region_list_response import RegionRegionListResponse
from openapi_client.models.response_billing_history_info import ResponseBillingHistoryInfo
from openapi_client.models.response_cli_driven_project import ResponseCLIDrivenProject
from openapi_client.models.response_daily_billing import ResponseDailyBilling
from openapi_client.models.response_dataset_info import ResponseDatasetInfo
from openapi_client.models.response_dataset_info_detail import ResponseDatasetInfoDetail
from openapi_client.models.response_dataset_info_list import ResponseDatasetInfoList
from openapi_client.models.response_dataset_source import ResponseDatasetSource
from openapi_client.models.response_dataset_summary import ResponseDatasetSummary
from openapi_client.models.response_dataset_version import ResponseDatasetVersion
from openapi_client.models.response_dataset_version_info import ResponseDatasetVersionInfo
from openapi_client.models.response_experiment_info import ResponseExperimentInfo
from openapi_client.models.response_experiment_list_response import ResponseExperimentListResponse
from openapi_client.models.response_experiment_plot_file import ResponseExperimentPlotFile
from openapi_client.models.response_file_metadata import ResponseFileMetadata
from openapi_client.models.response_kernel_cluster import ResponseKernelCluster
from openapi_client.models.response_kernel_cluster_info import ResponseKernelClusterInfo
from openapi_client.models.response_kernel_cluster_node import ResponseKernelClusterNode
from openapi_client.models.response_kernel_cluster_node_info import ResponseKernelClusterNodeInfo
from openapi_client.models.response_kernel_cluster_node_info_experiments_status_summary import ResponseKernelClusterNodeInfoExperimentsStatusSummary
from openapi_client.models.response_kernel_cluster_system_metrics import ResponseKernelClusterSystemMetrics
from openapi_client.models.response_kernel_image import ResponseKernelImage
from openapi_client.models.response_kernel_resource_spec import ResponseKernelResourceSpec
from openapi_client.models.response_my_user import ResponseMyUser
from openapi_client.models.response_organization import ResponseOrganization
from openapi_client.models.response_organization_credentials_info import ResponseOrganizationCredentialsInfo
from openapi_client.models.response_organization_credentials_info_list import ResponseOrganizationCredentialsInfoList
from openapi_client.models.response_organization_info import ResponseOrganizationInfo
from openapi_client.models.response_organization_member import ResponseOrganizationMember
from openapi_client.models.response_organization_notification_config import ResponseOrganizationNotificationConfig
from openapi_client.models.response_project import ResponseProject
from openapi_client.models.response_project_billing import ResponseProjectBilling
from openapi_client.models.response_project_experiment_summary import ResponseProjectExperimentSummary
from openapi_client.models.response_project_info import ResponseProjectInfo
from openapi_client.models.response_project_link import ResponseProjectLink
from openapi_client.models.response_project_list_response import ResponseProjectListResponse
from openapi_client.models.response_region import ResponseRegion
from openapi_client.models.response_ssh_key_info import ResponseSSHKeyInfo
from openapi_client.models.response_simple_experiment import ResponseSimpleExperiment
from openapi_client.models.response_simple_project import ResponseSimpleProject
from openapi_client.models.response_simple_user import ResponseSimpleUser
from openapi_client.models.response_simple_workspace import ResponseSimpleWorkspace
from openapi_client.models.response_sweep_experiment_summary import ResponseSweepExperimentSummary
from openapi_client.models.response_sweep_info import ResponseSweepInfo
from openapi_client.models.response_user import ResponseUser
from openapi_client.models.response_user_billing import ResponseUserBilling
from openapi_client.models.response_user_with_token_response import ResponseUserWithTokenResponse
from openapi_client.models.response_version_control_project import ResponseVersionControlProject
from openapi_client.models.response_volume import ResponseVolume
from openapi_client.models.response_volume_mount_info import ResponseVolumeMountInfo
from openapi_client.models.response_volume_mount_infos import ResponseVolumeMountInfos
from openapi_client.models.response_volume_source_dataset import ResponseVolumeSourceDataset
from openapi_client.models.response_volume_source_host_path import ResponseVolumeSourceHostPath
from openapi_client.models.response_volume_source_nfs import ResponseVolumeSourceNFS
from openapi_client.models.response_volume_source_project import ResponseVolumeSourceProject
from openapi_client.models.response_volume_source_volume import ResponseVolumeSourceVolume
from openapi_client.models.response_workload_endpoints import ResponseWorkloadEndpoints
from openapi_client.models.response_workload_history_info import ResponseWorkloadHistoryInfo
from openapi_client.models.response_workspace_detail import ResponseWorkspaceDetail
from openapi_client.models.response_workspace_list import ResponseWorkspaceList
from openapi_client.models.s3_dataset_create_api_payload import S3DatasetCreateAPIPayload
from openapi_client.models.ssh_key_create_api_payload import SSHKeyCreateAPIPayload
from openapi_client.models.savvi_hub_dataset_create_api_payload import SavviHubDatasetCreateAPIPayload
from openapi_client.models.sign_in_api_payload import SignInAPIPayload
from openapi_client.models.sign_in_cli_confirm_api_payload import SignInCliConfirmAPIPayload
from openapi_client.models.sign_in_git_hub_api_payload import SignInGitHubAPIPayload
from openapi_client.models.sign_in_google_api_payload import SignInGoogleAPIPayload
from openapi_client.models.sign_up_api_payload import SignUpAPIPayload
from openapi_client.models.sign_up_validate_api_payload import SignUpValidateAPIPayload
from openapi_client.models.slack_authorize_api_payload import SlackAuthorizeAPIPayload
from openapi_client.models.storage_federation_token import StorageFederationToken
from openapi_client.models.storage_file_action_url_info import StorageFileActionURLInfo
from openapi_client.models.sweep_create_api_payload import SweepCreateAPIPayload
from openapi_client.models.sweep_run_api_payload import SweepRunAPIPayload
from openapi_client.models.sweep_sweep_list_response import SweepSweepListResponse
from openapi_client.models.verify_email_api_payload import VerifyEmailAPIPayload
from openapi_client.models.volume_file_copy_api_payload import VolumeFileCopyAPIPayload
from openapi_client.models.volume_file_create_api_payload import VolumeFileCreateAPIPayload
from openapi_client.models.volume_volume_file_copy_response import VolumeVolumeFileCopyResponse
from openapi_client.models.volume_volume_file_delete_response import VolumeVolumeFileDeleteResponse
from openapi_client.models.volume_volume_file_list_response import VolumeVolumeFileListResponse
from openapi_client.models.workspace_create_api_payload import WorkspaceCreateAPIPayload
from openapi_client.models.workspace_update_api_payload import WorkspaceUpdateAPIPayload
from openapi_client.models.workspace_workspace_list_response import WorkspaceWorkspaceListResponse
from openapi_client.models.workspace_workspace_logs_response import WorkspaceWorkspaceLogsResponse
from openapi_client.models.workspace_workspace_system_metrics_response import WorkspaceWorkspaceSystemMetricsResponse



import os
from typing import Optional, Union

import six
from PIL import Image as PIL_Image

from savvihub.constants import SAVVIHUB_IMAGES
from savvihub.utils import get_type_name, generate_uuid, to_uint8


class Image:
    def __init__(
        self,
        data: Union[str, "Image", "PIL.Image", "np.ndarray"],
        caption: Optional[str] = None,
        mode: Optional[str] = None
    ):
        self._image = None
        self._caption = None
        self._mode = None
        self._path = None

        self._init_meta(caption, mode)
        self._init_image(data)
        self._save_image()

    def _init_meta(self, caption, mode):
        self._caption = caption
        self._mode = mode

        if not os.path.exists(SAVVIHUB_IMAGES):
            os.makedirs(SAVVIHUB_IMAGES)

        self._path = os.path.join(SAVVIHUB_IMAGES, generate_uuid() + '.png')

    def _init_image_from_path(self, data):
        if not os.path.isfile(data):
            print(f'Error: {data} file not found')
        self._image = PIL_Image.open(data)

    def _init_image_from_data(self, data):
        if get_type_name(data).startswith("torch."):
            self._image = PIL_Image.fromarray(
                data.mul(255).clamp(0, 255).byte().permute(1, 2, 0).squeeze().cpu().numpy()
            )
        elif isinstance(data, PIL_Image.Image):
            self._image = data
        else:
            if hasattr(data, "numpy"):
                data = data.numpy()
            if data.ndim > 2:
                data = data.squeeze()
            self._image = PIL_Image.fromarray(
                to_uint8(data), mode=self._mode,
            )

    def _init_image(self, data):
        if isinstance(data, Image):
            self._image = data._image
        elif isinstance(data, six.string_types):
            self._init_image_from_path(data)
        else:
            self._init_image_from_data(data)

    def _save_image(self):
        self._image.save(self._path)

    def flush(self):
        if os.path.isfile(self._path):
            os.remove(self._path)
        else:
            print(f'Error: {self._path} file not found')

    def get_path(self):
        return self._path

    def get_caption(self):
        return self._caption



import inquirer
import typer
from savvihub.api.exceptions import InvalidParametersAPIException, DuplicateAPIException

from openapi_client import ResponseOrganization
from savvihub import SavviHubClient


def get_default_organization(client: SavviHubClient) -> ResponseOrganization:
    organizations = client.organization_list().organizations
    if len(organizations) == 0:
        region_list_resp = client.region_list()
        typer.echo('Create organization')
        while True:
            organization_name = inquirer.prompt([inquirer.Text(
                'organization_name',
                message="Organization name",
            )], raise_keyboard_interrupt=True).get('organization_name')
            region = inquirer.prompt([inquirer.List(
                'region',
                message="Select region",
                default=region_list_resp.default_region,
                choices=[(region.name, region.value) for region in region_list_resp.regions]
            )], raise_keyboard_interrupt=True).get('region')
            try:
                default_organization = client.organization_create(organization_name=organization_name, region=region)
                break
            except InvalidParametersAPIException:
                typer.echo('Invalid organization name. Please try again.')
            except DuplicateAPIException:
                typer.echo('Duplicate organization name exist. Please try again.')
    elif len(organizations) == 1:
        default_organization = organizations[0]
        typer.echo(f'Default organization is automatically set to `{default_organization.name}`.')
    else:
        default_organization = inquirer.prompt([inquirer.List(
            'default_organization',
            message='Select default organization',
            choices=[(ws.name, ws) for ws in organizations],
        )], raise_keyboard_interrupt=True).get('default_organization')

        typer.echo(f'Default organization is set to `{default_organization.name}`.')

    return default_organization


def find_from_inquirer(options, display, message):
    return inquirer.prompt([inquirer.List(
        "question",
        message=message,
        choices=[(f'[{i+1}] {display(option)}', option) for i, option in enumerate(options)],
    )], raise_keyboard_interrupt=True).get("question")


def parse_dataset(dataset_full_name):
    if '@' in dataset_full_name:
        dataset_name, dataset_version_name = dataset_full_name.split('@', 1)
    else:
        dataset_name = dataset_full_name
        dataset_version_name = 'latest'

    if '/' in dataset_name:
        organization_name, dataset_name = dataset_name.split('/', 1)
    else:
        organization_name = None

    return organization_name, dataset_name, dataset_version_name
